@@ -1,6 +1,6 @@
 /* gateway_commands.c
  *
- * $Id: gateway_commands.c,v 1.5 2000/11/15 16:41:06 nisse Exp $
+ * $Id: gateway_commands.c,v 1.8 2001/02/25 22:38:18 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -28,6 +28,7 @@
 #include "connection_commands.h"
 #include "debug.h"
 #include "format.h"
+#include "gateway_channel.h"
 #include "io_commands.h"
 #include "read_packet.h"
 #include "ssh.h"
@@ -39,7 +40,7 @@
 
 #include "gateway_commands.c.x"
 
-/* A simplified version of do_pad() in pad.c. This one uses
+/* A simplified version of do_pad in pad.c. This one uses
  * a fixed block size and pads with zeros. */
 
 /* FIXME: It seems very unnecessary to pad at all; we could just use
@@ -111,7 +112,7 @@ gateway_make_connection(struct listen_value *lv,
 			  NULL, /* established_continuation */
 			  make_exc_finish_read_handler(lv->fd, e, HANDLER_CONTEXT));
 
-  /* Instead of calling connection_init_io(). */
+  /* Instead of calling connection_init_io. */
   connection->raw =
     &io_read_write(lv->fd,
 		   make_buffered_read(
@@ -129,11 +130,13 @@ gateway_make_connection(struct listen_value *lv,
 			make_gateway_pad(connection->raw),
 			ssh_format("%lz sent", connection->debug_comment));
 
+  init_connection_service(connection);
+
+  connection->table->open_fallback = &gateway_channel_open_forward;
+
   connection->dispatch[SSH_MSG_DEBUG] = &connection_forward_handler;
   connection->dispatch[SSH_MSG_IGNORE] = &connection_forward_handler;
 
-  init_connection_service(connection);
-
   return connection;
 }
 
@@ -147,14 +150,6 @@ DEFINE_COMMAND(gateway_init, a, c, e)
 
 /* (gateway_accept main-connection gateway-connection) */
 
-/* ;; GABA:
-   (class
-     (name gateway_accept_command)
-     (super command)
-     (vars
-       (connection object ssh_connection)))
-*/
-
 static void
 do_gateway_accept(struct command *s,
 		 struct lsh_object *x,
@@ -217,4 +212,3 @@ DEFINE_COMMAND_SIMPLE(gateway_setup_command, a)
 
   return &res->super;
 }
-
