@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: io.h,v 1.74 2001/03/12 21:22:55 nisse Exp $ */
+ * $Id: io.h,v 1.75 2001/03/14 18:26:13 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -312,12 +312,6 @@ make_listen_callback(struct io_backend *backend,
 		     struct command_continuation *c,
 		     struct exception_handler *e);
 
-#if 0
-struct io_callback *
-make_listen_callback_no_peer(struct io_backend *backend,
-			     struct command_continuation *c);
-#endif
-
 struct lsh_fd *io_read_write(struct lsh_fd *fd,
 			     struct io_callback *read,
 			     UINT32 block_size,
@@ -331,7 +325,6 @@ struct lsh_fd *io_write(struct lsh_fd *fd,
 			UINT32 block_size,
 			struct lsh_callback *close_callback);
 
-/* Marks a file for close, without touching the close_reason field. */
 void kill_fd(struct lsh_fd *fd);
 
 void close_fd(struct lsh_fd *fd);
@@ -343,12 +336,13 @@ void close_fd_nicely(struct lsh_fd *fd);
 /* Stop reading, but if the fd has a write callback, keep it open. */
 void close_fd_read(struct lsh_fd *fd);
 
-struct lsh_fd *io_write_file(struct io_backend *backend,
-			    const char *fname, int flags,
-			    int mode,
-			    UINT32 block_size,
-			    struct lsh_callback *c,
-			    struct exception_handler *e);
+struct lsh_fd *
+io_write_file(struct io_backend *backend,
+	      const char *fname, int flags,
+	      int mode,
+	      UINT32 block_size,
+	      struct lsh_callback *c,
+	      struct exception_handler *e);
 
 struct lsh_fd *
 io_read_file(struct io_backend *backend,
