@@ -2,7 +2,7 @@
  *
  * main server program.
  *
- * $Id: lshd.c,v 1.105 2000/11/26 18:09:14 nisse Exp $ */
+ * $Id: lshd.c,v 1.112 2001/03/25 12:23:32 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -95,7 +95,12 @@ const char *argp_program_version
 
 const char *argp_program_bug_address = BUG_ADDRESS;
 
-#define KERBEROS_HELPER PREFIX "lsh-krb-checkpw"
+/* The definition of SBINDIR is currently broken */
+#if 0
+# define KERBEROS_HELPER SBINDIR "/lsh-krb-checkpw"
+#else
+# define KERBEROS_HELPER PREFIX "/sbin/lsh-krb-checkpw"
+#endif
 
 #define OPT_NO 0x400
 #define OPT_SSH1_FALLBACK 0x200
@@ -130,6 +135,8 @@ const char *argp_program_bug_address = BUG_ADDRESS;
 
 #define OPT_PASSWORD_HELPER 0x224
 
+#define OPT_LOGIN_SHELL 0x225
+
 /* GABA:
    (class
      (name lshd_options)
@@ -158,6 +165,7 @@ const char *argp_program_bug_address = BUG_ADDRESS;
        (with_password . int)
        (allow_root . int)
        (pw_helper . "const char *")
+       (login_shell . "const char *")
        
        (with_tcpip_forward . int)
        (with_pty . int)
@@ -232,6 +240,7 @@ make_lshd_options(struct io_backend *backend)
   self->with_pty = 1;
   self->allow_root = 0;
   self->pw_helper = NULL;
+  self->login_shell = NULL;
   
   self->userauth_methods = NULL;
   self->userauth_algorithms = NULL;
@@ -327,15 +336,19 @@ main_options[] =
   { "no-root-login", OPT_NO_ROOT_LOGIN, NULL, 0,
     "Don't allow root to login (default).", 0 },
 
+  { "login-shell", OPT_LOGIN_SHELL, "Program", 0,
+    "Use this program as the login shell for all users. "
+    "(Experimental)", 0 },
+  
   { "kerberos-passwords", OPT_KERBEROS_PASSWD, NULL, 0,
     "Recognize kerberos passwords, using the helper program "
     "\"" KERBEROS_HELPER "\". This option is experimental.", 0 },
   { "no-kerberos-passwords", OPT_NO_KERBEROS_PASSWD, NULL, 0,
-    "Don't recognize kerberos passwords (default behaviour)." },
+    "Don't recognize kerberos passwords (default behaviour).", 0 },
 
   { "password-helper", OPT_PASSWORD_HELPER, "Program", 0,
     "Use the named helper program for password verification. "
-    "(experimental).", 0 },
+    "(Experimental).", 0 },
   
   { NULL, 0, NULL, 0, "Offered services:", 0 },
 
@@ -380,11 +393,12 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
       break;
     case ARGP_KEY_END:
       {
-	struct user_db *db = NULL;
+	struct user_db *user_db = NULL;
 	
 	if (self->with_password || self->with_publickey || self->with_srp_keyexchange)
-	  db = make_unix_user_db(self->backend, self->reaper,
-				 self->pw_helper, self->allow_root);
+	  user_db = make_unix_user_db(self->backend, self->reaper,
+				      self->pw_helper, self->login_shell,
+				      self->allow_root);
 	  
 	if (self->with_dh_keyexchange || self->with_srp_keyexchange)
 	  {
@@ -403,11 +417,12 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 #if WITH_SRP	    
 	    if (self->with_srp_keyexchange)
 	      {
-		assert(db);
+		assert(user_db);
 		LIST(self->kex_algorithms)[i++] = ATOM_SRP_RING1_SHA1_LOCAL;
 		ALIST_SET(self->super.algorithms,
 			  ATOM_SRP_RING1_SHA1_LOCAL,
-			  &make_srp_server(make_srp1(&self->random->super), db)
+			  &make_srp_server(make_srp1(&self->random->super),
+					   user_db)
 			  ->super);
 	      }
 #endif /* WITH_SRP */
@@ -441,20 +456,23 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 		LIST(self->userauth_methods)[i++] = ATOM_PASSWORD;
 		ALIST_SET(self->userauth_algorithms,
 			  ATOM_PASSWORD,
-			  &make_userauth_password(db)->super);
+			  &make_userauth_password(user_db)->super);
 	      }
 	    if (self->with_publickey)
 	      {
-		/* Doesn't use spki */
+		/* FIXME: Doesn't use spki */
+		struct lookup_verifier *key_db
+		  = make_authorization_db(ssh_format("authorized_keys_sha1"),
+					  &sha1_algorithm);
+		
 		LIST(self->userauth_methods)[i++] = ATOM_PUBLICKEY;
 		ALIST_SET(self->userauth_algorithms,
 			  ATOM_PUBLICKEY,
 			  &make_userauth_publickey
-			  (db,
-			   make_alist(1,
-				      ATOM_SSH_DSS,
-				      make_authorization_db(ssh_format("authorized_keys_sha1"),
-							    &sha1_algorithm),
+			  (user_db,
+			   make_alist(2,
+				      ATOM_SSH_DSS, key_db,
+				      ATOM_SSH_RSA, key_db,
 				      -1))
 			  ->super);
 	      }
@@ -532,6 +550,10 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
     case OPT_PASSWORD_HELPER:
       self->pw_helper = arg;
       break;
+
+    case OPT_LOGIN_SHELL:
+      self->login_shell = arg;
+      break;
       
 #if WITH_TCP_FORWARD
     case OPT_TCPIP_FORWARD:
@@ -626,6 +648,50 @@ main_argp =
 	      (connection_require_userauth connection)))))))
 */
 
+#if WITH_GCOV
+/* FIXME: Perhaps move to daemon.c? */
+/* Catch SIGTERM and call exit(). That way, profiling info is written
+ * properly when the process is terminated. */
+
+static volatile sig_atomic_t terminate;
+
+static void terminate_handler(int signum)
+{
+  assert(signum == SIGTERM);
+
+  terminate = 1;
+}
+
+static void
+do_terminate_callback(struct lsh_callback *s UNUSED)
+{
+  gc_final();
+  exit(0);
+}
+
+static struct lsh_callback
+terminate_callback =
+{ STATIC_HEADER, do_terminate_callback };
+
+static void
+install_terminate_handler(struct io_backend *backend)
+{
+  struct sigaction term;
+  memset(&term, 0, sizeof(term));
+
+  term.sa_handler = terminate_handler;
+  sigemptyset(&term.sa_mask);
+  term.sa_flags = 0;
+
+  if (sigaction(SIGTERM, &term, NULL) < 0)
+    {
+      werror ("Failed to install SIGTERM handler (errno = %i): %z\n",
+	      errno, STRERROR(errno));
+      exit(EXIT_FAILURE);
+    }
+  io_signal_handler(backend, &terminate, &terminate_callback);
+}
+#endif /* WITH_GCOV */
 
 int main(int argc, char **argv)
 {
@@ -633,6 +699,10 @@ int main(int argc, char **argv)
 
   struct io_backend *backend = make_io_backend();
 
+#if WITH_GCOV
+  install_terminate_handler(backend);
+#endif
+  
   /* For filtering messages. Could perhaps also be used when converting
    * strings to and from UTF8. */
   setlocale(LC_CTYPE, "");
@@ -762,5 +832,7 @@ int main(int argc, char **argv)
   
   io_run(backend);
 
+  gc_final();
+  
   return 0;
 }
