@@ -1,6 +1,6 @@
 /* lshg.c
  *
- * $Id: lshg.c,v 1.6 2001/02/18 22:29:48 nisse Exp $
+ * $Id: lshg.c,v 1.8 2001/03/25 12:24:23 nisse Exp $
  *
  * Connect to a gateway. */
 
@@ -24,7 +24,7 @@
  */
 
 #include "charset.h"
-#include "client.c"
+#include "client.h"
 #include "connection.h"
 #include "debug.h"
 #include "format.h"
@@ -278,6 +278,8 @@ main(int argc, char **argv)
   }
 
   io_run(backend);
+
+  gc_final();
   
   return exit_code;
 }
