@@ -2,7 +2,7 @@
  *
  * User-related functions on UN*X
  *
- * $Id: unix_user.c,v 1.30 2000/11/16 22:46:57 nisse Exp $
+ * $Id: unix_user.c,v 1.32 2001/02/25 22:38:22 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -204,7 +204,7 @@ do_kill_process(struct resource *r)
 
       if (kill(self->pid, self->signal) < 0)
 	{
-	  werror("do_kill_process: kill() failed (errno = %i): %z\n",
+	  werror("do_kill_process: kill failed (errno = %i): %z\n",
 		 errno, STRERROR(errno));
 	}
     }
@@ -313,7 +313,7 @@ kerberos_check_pw(struct unix_user *user, struct lsh_string *pw,
   switch (child)
     {
     case -1:
-      werror("kerberos_check_pw: fork() failed: %z\n", STRERROR(errno));
+      werror("kerberos_check_pw: fork failed: %z\n", STRERROR(errno));
       return 0;
 
     case 0:
@@ -412,7 +412,7 @@ do_verify_password(struct lsh_user *s,
   {
     char *salt = user->passwd->data;
 
-    /* NOTE: crypt() uses the *ugly* convention of returning pointers
+    /* NOTE: crypt uses the *ugly* convention of returning pointers
      * to static buffers. */
 
     if (strcmp(crypt(password->data, salt), user->passwd->data))
@@ -606,7 +606,7 @@ do_read_file(struct lsh_user *u,
 	
 	if ( (me != user->super.uid) && (setuid(user->super.uid) < 0) )
 	  {
-	    werror("unix_user.c: do_read_file(): setuid failed (errno = %i): %z\n",
+	    werror("unix_user.c: do_read_file: setuid failed (errno = %i): %z\n",
 		   errno, STRERROR(errno));
 	    _exit(EXIT_FAILURE);
 	  }
@@ -619,7 +619,7 @@ do_read_file(struct lsh_user *u,
 
 	if (fstat(fd, &sbuf) < 0)
 	  {
-	    werror("unix_user.c: do_read_file(): fstat failed (errno = %i): %z\n",
+	    werror("unix_user.c: do_read_file: fstat failed (errno = %i): %z\n",
 		   errno, STRERROR(errno));
 	    _exit(EXIT_FAILURE);
 	  }
@@ -628,7 +628,7 @@ do_read_file(struct lsh_user *u,
 
 	if (x)
 	  {
-	    werror("unix_user.c: do_read_file(): %z\n", x->msg);
+	    werror("unix_user.c: do_read_file: %z\n", x->msg);
 	    _exit(EXIT_FAILURE);
 	  }
 
@@ -720,7 +720,7 @@ do_fork_process(struct lsh_user *u,
   switch(child)
     {
     case -1: 
-      werror("fork() failed: %z\n", STRERROR(errno));
+      werror("fork failed: %z\n", STRERROR(errno));
       return 0;
 
     case 0: /* Child */
@@ -893,7 +893,13 @@ make_unix_user(struct lsh_string *name,
      (vars
        (backend object io_backend)
        (reaper object reap)
+
+       ; A program to use for verifying passwords.
        (pw_helper . "const char *")
+
+       ; Override the login shell for all users.
+       (login_shell . "const char *")
+       
        (allow_root . int)))
 */
 
@@ -916,6 +922,7 @@ do_lookup_user(struct user_db *s,
   
   struct passwd *passwd;
   const char *home;
+  const char *shell;
   
   name = make_cstring(name, free);
   
@@ -994,12 +1001,18 @@ do_lookup_user(struct user_db *s,
 	     && (passwd->pw_uid == getuid())
 	     && (home = getenv("HOME"))))
 	home = passwd->pw_dir;
+
+      if (self->login_shell)
+	/* Override the passwd database */
+	shell = self->login_shell;
+      else
+	shell = passwd->pw_shell;
       
       return make_unix_user(name, 
 			    passwd->pw_uid, passwd->pw_gid,
 			    self,
 			    crypted,
-			    home, passwd->pw_shell);
+			    home, shell);
     }
   else
     {
@@ -1011,7 +1024,7 @@ do_lookup_user(struct user_db *s,
 
 struct user_db *
 make_unix_user_db(struct io_backend *backend, struct reap *reaper,
-		  const char *pw_helper,
+		  const char *pw_helper, const char *login_shell,
 		  int allow_root)
 {
   NEW(unix_user_db, self);
@@ -1020,6 +1033,7 @@ make_unix_user_db(struct io_backend *backend, struct reap *reaper,
   self->backend = backend;
   self->reaper = reaper;
   self->pw_helper = pw_helper;
+  self->login_shell = login_shell;
   self->allow_root = allow_root;
 
   return &self->super;
