@@ -2,7 +2,7 @@
  *
  * zlib compression algorithm
  * 
- * $Id: zlib.c,v 1.29 2000/11/26 23:26:04 nisse Exp $ */
+ * $Id: zlib.c,v 1.30 2001/02/25 22:38:22 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -213,7 +213,7 @@ do_zlib(struct compress_instance *c,
 	case Z_OK:
 	  break;
 	default:
-	  werror("do_zlib: %z() failed: %z\n",
+	  werror("do_zlib: %z failed: %z\n",
 		 ZLIB_TYPE(&self->z)->operation,
 		 self->z.msg ? self->z.msg : "No error(?)");
 	  if (free)
